import java.util.Arrays;

public class AveragingFilter implements PixelBufferFilter2 {
	protected PixelBufferFilter2[] filters;
	private int[][] pixelBuffers;
	
	// ### INIT ###
	
	public AveragingFilter(PixelBufferFilter2[] filters) {
		this.filters = filters;
		this.pixelBuffers = new int[filters.length][];
	}
	
	// ### FILTER ###
	
	public void filter(int width, int height, int[] inPixels, int[] outPixels) {
		// Ensure intermediate pixel buffers are of the correct size
		int numPixels = inPixels.length;
		if ((pixelBuffers[0] == null) || (pixelBuffers[0].length != numPixels)) {
			for (int i=0, n=pixelBuffers.length; i<n; i++) {
				pixelBuffers[i] = new int[numPixels];
			}
		}
		
		// Execute intermediate filters
		for (int i=0, n=pixelBuffers.length; i<n; i++) {
			filters[i].filter(width, height, inPixels, pixelBuffers[i]);
		}
		
		// Average the results of all the filters
		int numFilters = filters.length;
		for (int off=0, n=inPixels.length; off<n; off++) {
			int alphaSum = 0;
			int redSum = 0;
			int greenSum = 0;
			int blueSum = 0;
			for (int i=0; i<numFilters; i++) {
				int curPixel = pixelBuffers[i][off];
				alphaSum += Pixels.getAlpha(curPixel);
				redSum += Pixels.getRed(curPixel);
				greenSum += Pixels.getGreen(curPixel);
				blueSum += Pixels.getBlue(curPixel);
			}
			
			int avgAlpha = alphaSum/numFilters;
			int avgRed = redSum/numFilters;
			int avgGreen = greenSum/numFilters;
			int avgBlue = blueSum/numFilters;
			outPixels[off] = Pixels.packPixel(avgAlpha, avgRed, avgGreen, avgBlue);
		}
	}
	
	public String toString() {
		return "Average "+Arrays.asList(filters);
	}
}